@@ -68,6 +68,7 @@ static GType
 invoke_get_type (GModule *self, const char *symbol, GError **error)
 {
   GetTypeFunc sym;
+  GType ret;
 
   if (!g_module_symbol (self, symbol, (void**)&sym))
     {
@@ -78,7 +79,15 @@ invoke_get_type (GModule *self, const char *symbol, GError **error)
       return G_TYPE_INVALID;
     }
 
-  return sym ();
+  ret = sym ();
+  if (ret == G_TYPE_INVALID)
+    {
+      g_set_error (error,
+		   G_IO_ERROR,
+		   G_IO_ERROR_FAILED,
+		   "Function '%s' returned G_TYPE_INVALID", symbol);
+    }
+  return ret;
 }
 
 static void
@@ -159,17 +168,17 @@ dump_object_type (GType type, const char *symbol, GOutputStream *out)
       GType parent;
       gboolean first = TRUE;
 
-      parent = type;
+      parent = g_type_parent (type);
       parent_str = g_string_new ("");
-      do
+      while (parent != G_TYPE_OBJECT && parent != G_TYPE_INVALID)
         {
-          parent = g_type_parent (parent);
           if (first)
             first = FALSE;
           else
             g_string_append_c (parent_str, ',');
           g_string_append (parent_str, g_type_name (parent));
-        } while (parent != G_TYPE_OBJECT && parent != G_TYPE_INVALID);
+          parent = g_type_parent (parent);
+        }
 
       escaped_printf (out, " parents=\"%s\"", parent_str->str);
 
@@ -270,6 +279,56 @@ dump_enum_type (GType type, const char *symbol, GOutputStream *out)
   goutput_write (out, "  </enum>");
 }
 
+static void
+dump_fundamental_type (GType type, const char *symbol, GOutputStream *out)
+{
+  guint n_interfaces;
+  guint i;
+  GType *interfaces;
+  GString *parent_str;
+  GType parent;
+  gboolean first = TRUE;
+
+
+  escaped_printf (out, "  <fundamental name=\"%s\" get-type=\"%s\"",
+		  g_type_name (type), symbol);
+
+  if (G_TYPE_IS_ABSTRACT (type))
+    escaped_printf (out, " abstract=\"1\"");
+
+  if (G_TYPE_IS_INSTANTIATABLE (type))
+    escaped_printf (out, " instantiatable=\"1\"");
+
+  parent = g_type_parent (type);
+  parent_str = g_string_new ("");
+  while (parent != G_TYPE_INVALID)
+    {
+      if (first)
+        first = FALSE;
+      else
+        g_string_append_c (parent_str, ',');
+      if (!g_type_name (parent))
+        break;
+      g_string_append (parent_str, g_type_name (parent));
+      parent = g_type_parent (parent);
+    }
+
+  if (parent_str->len > 0)
+    escaped_printf (out, " parents=\"%s\"", parent_str->str);
+  g_string_free (parent_str, TRUE);
+
+  goutput_write (out, ">\n");
+
+  interfaces = g_type_interfaces (type, &n_interfaces);
+  for (i = 0; i < n_interfaces; i++)
+    {
+      GType itype = interfaces[i];
+      escaped_printf (out, "    <implements name=\"%s\"/>\n",
+		      g_type_name (itype));
+    }
+  goutput_write (out, "  </fundamental>\n");
+}
+
 static void
 dump_type (GType type, const char *symbol, GOutputStream *out)
 {
@@ -294,10 +353,7 @@ dump_type (GType type, const char *symbol, GOutputStream *out)
       /* GValue, etc.  Just skip them. */
       break;
     default:
-      /* Other fundamental types such as the once GStreamer and Clutter registers
-       * are not yet interesting from an introspection perspective and should be
-       * ignored
-       */
+      dump_fundamental_type (type, symbol, out);
       break;
     }
 }
@@ -382,7 +438,6 @@ g_irepository_dump (const char *arg, GError **error)
 
       if (type == G_TYPE_INVALID)
 	{
-          g_printerr ("Invalid GType: '%s'\n", line);
           caught_error = TRUE;
 	  g_free (line);
 	  break;
