@@ -22,13 +22,14 @@ import re
 from . import ast
 from . import message
 from .annotationparser import (TAG_VFUNC, TAG_SINCE, TAG_DEPRECATED, TAG_RETURNS,
-                               TAG_ATTRIBUTES, TAG_RENAME_TO, TAG_TYPE, TAG_TRANSFER,
+                               TAG_ATTRIBUTES, TAG_RENAME_TO, TAG_TYPE,
                                TAG_UNREF_FUNC, TAG_REF_FUNC, TAG_SET_VALUE_FUNC,
                                TAG_GET_VALUE_FUNC)
-from .annotationparser import (OPT_ALLOW_NONE,
-                               OPT_ARRAY, OPT_ELEMENT_TYPE, OPT_IN, OPT_INOUT,
+from .annotationparser import (OPT_ALLOW_NONE, OPT_ARRAY, OPT_ATTRIBUTE,
+                               OPT_ELEMENT_TYPE, OPT_IN, OPT_INOUT,
                                OPT_INOUT_ALT, OPT_OUT, OPT_SCOPE,
-                               OPT_TYPE, OPT_CLOSURE, OPT_DESTROY, OPT_SKIP,
+                               OPT_OUT_CALLER_ALLOCATES, OPT_OUT_CALLEE_ALLOCATES,
+                               OPT_TYPE, OPT_CLOSURE, OPT_DESTROY, OPT_TRANSFER, OPT_SKIP,
                                OPT_FOREIGN, OPT_ARRAY_FIXED_SIZE,
                                OPT_ARRAY_LENGTH, OPT_ARRAY_ZERO_TERMINATED)
 from .annotationparser import AnnotationParser
@@ -65,6 +66,9 @@ class MainTransformer(object):
         # Type() types and see if they match up with something.
         self._namespace.walk(self._pass_type_resolution)
 
+        # Read in annotations needed early
+        self._namespace.walk(self._pass_read_annotations_early)
+
         # Determine some default values for transfer etc.
         # based on the current tree.
         self._namespace.walk(self._pass_callable_defaults)
@@ -111,6 +115,8 @@ usage is void (*_gtk_reserved1)(void);"""
                                  ast.Record, ast.Union)):
             return True
         for field in node.fields:
+            if field is None:
+                continue
             if (field.name.startswith('_')
                 and field.anonymous_node is not None
                 and isinstance(field.anonymous_node, ast.Callback)):
@@ -166,6 +172,15 @@ usage is void (*_gtk_reserved1)(void);"""
         block = self._blocks.get(node.symbol)
         self._apply_annotations_callable(node, chain, block)
 
+    def _pass_read_annotations_early(self, node, chain):
+        if isinstance(node, ast.Record):
+            if node.ctype is not None:
+                block = self._blocks.get(node.ctype)
+            else:
+                block = self._blocks.get(node.c_name)
+            self._apply_annotations_annotated(node, block)
+        return True
+
     def _pass_callable_defaults(self, node, chain):
         if isinstance(node, (ast.Callable, ast.Signal)):
             for param in node.parameters:
@@ -184,7 +199,7 @@ usage is void (*_gtk_reserved1)(void);"""
             elif isinstance(node, ast.Registered) and node.gtype_name is not None:
                 return node.gtype_name
             return node.c_name
-        assert False, "Unhandled node %r" % (node, )
+        raise AssertionError("Unhandled node %r" % (node, ))
 
     def _get_block(self, node):
         return self._blocks.get(self._get_annotation_name(node))
@@ -198,9 +213,8 @@ usage is void (*_gtk_reserved1)(void);"""
             self._apply_annotations_function(node, chain)
         if isinstance(node, ast.Callback):
             self._apply_annotations_callable(node, chain, block = self._get_block(node))
-        if isinstance(node, (ast.Class, ast.Interface, ast.Record,
-                             ast.Union, ast.Enum, ast.Bitfield,
-                             ast.Callback)):
+        if isinstance(node, (ast.Class, ast.Interface, ast.Union, ast.Enum,
+                             ast.Bitfield, ast.Callback)):
             self._apply_annotations_annotated(node, self._get_block(node))
         if isinstance(node, (ast.Class, ast.Interface, ast.Record, ast.Union)):
             block = self._get_block(node)
@@ -233,7 +247,7 @@ usage is void (*_gtk_reserved1)(void);"""
         elif has_element_type:
             self._apply_annotations_element_type(parent, node, options)
 
-    def _resolve(self, type_str, orig_node=None):
+    def _resolve(self, type_str, type_node=None, node=None, parent=None):
         def grab_one(type_str, resolver, top_combiner, combiner):
             """Return a complete type, and the trailing string part after it.
             Use resolver() on each identifier, and combiner() on the parts of
@@ -263,8 +277,8 @@ usage is void (*_gtk_reserved1)(void);"""
                 "Too many parameters in type specification %r" % (type_str, ))
             return base
         def top_combiner(base, *rest):
-            if orig_node is not None and isinstance(orig_node, ast.Type):
-                base.is_const = orig_node.is_const
+            if type_node is not None and isinstance(type_node, ast.Type):
+                base.is_const = type_node.is_const
             return combiner(base, *rest)
 
         result, rest = grab_one(type_str, resolver, top_combiner, combiner)
@@ -273,15 +287,31 @@ usage is void (*_gtk_reserved1)(void);"""
                 type_str, ))
 
         if not result.resolved:
-            parent = orig_node
-            if isinstance(parent, ast.Function):
+            position = None
+            if parent is not None and isinstance(parent, ast.Function):
                 text = parent.symbol
+                position = self._get_position(parent, node)
             else:
-                text = parent.name
+                text = type_str
             message.warn_node(parent, "%s: Unknown type: %r" %
-                              (text, result.ctype))
+                              (text, result.ctype), positions=position)
         return result
 
+    def _get_position(self, func, param):
+        block = self._blocks.get(func.symbol)
+        if block:
+            if isinstance(param, ast.Parameter):
+                tag = block.tags.get(param.argname)
+            elif isinstance(param, ast.Return):
+                tag = block.tags.get(TAG_RETURNS)
+            else:
+                tag = None
+
+            if tag.position:
+                return tag.position
+
+        return block.position
+
     def _apply_annotations_array(self, parent, node, options):
         array_opt = options.get(OPT_ARRAY)
         if array_opt:
@@ -291,7 +321,8 @@ usage is void (*_gtk_reserved1)(void);"""
 
         element_type = options.get(OPT_ELEMENT_TYPE)
         if element_type is not None:
-            element_type_node = self._resolve(element_type.one(), parent)
+            element_type_node = self._resolve(element_type.one(),
+                                              node.type, node, parent)
         elif isinstance(node.type, ast.Array):
             element_type_node = node.type.element_type
         else:
@@ -322,21 +353,52 @@ usage is void (*_gtk_reserved1)(void);"""
                 container_type.length_param_name = param.argname
         fixed = array_values.get(OPT_ARRAY_FIXED_SIZE)
         if fixed:
-            container_type.size = int(fixed)
+            try:
+                container_type.size = int(fixed)
+            except ValueError:
+                # Already warned in annotationparser.py
+                return
         node.type = container_type
 
     def _apply_annotations_element_type(self, parent, node, options):
         element_type_opt = options.get(OPT_ELEMENT_TYPE)
-        element_type = element_type_opt.flat()
+        if element_type_opt is None:
+            message.warn(
+                'element-type annotation takes at least one option, '
+                'none given',
+                options.position)
+            return
+
         if isinstance(node.type, ast.List):
-            assert len(element_type) == 1
-            node.type.element_type = self._resolve(element_type[0], parent)
+            if element_type_opt.length() != 1:
+                message.warn(
+                    'element-type annotation for a list must have exactly '
+                    'one option, not %d options' % (element_type_opt.length(), ),
+                    options.position)
+                return
+            node.type.element_type = self._resolve(element_type_opt.one(),
+                                                   node.type, node, parent)
         elif isinstance(node.type, ast.Map):
-            assert len(element_type) == 2
-            node.type.key_type = self._resolve(element_type[0], parent)
-            node.type.value_type = self._resolve(element_type[1], parent)
+            if element_type_opt.length() != 2:
+                message.warn(
+                    'element-type annotation for a hash table must have exactly '
+                    'two options, not %d option(s)' % (element_type_opt.length(), ),
+                    options.position)
+                return
+            element_type = element_type_opt.flat()
+            node.type.key_type = self._resolve(element_type[0],
+                                               node.type, node, parent)
+            node.type.value_type = self._resolve(element_type[1],
+                                                 node.type, node, parent)
         elif isinstance(node.type, ast.Array):
-            node.type.element_type = self._resolve(element_type[0], parent)
+            if element_type_opt.length() != 1:
+                message.warn(
+                    'element-type annotation for an array must have exactly '
+                    'one option, not %d options' % (element_type_opt.length(), ),
+                    options.position)
+                return
+            node.type.element_type = self._resolve(element_type_opt.one(),
+                                                   node.type, node, parent)
         else:
             message.warn_node(parent,
                 "Unknown container %r for element-type annotation" % (node.type, ))
@@ -384,7 +446,8 @@ usage is void (*_gtk_reserved1)(void);"""
         if isinstance(target, ast.Alias):
             return self._get_transfer_default_returntype_basic(target.target)
         elif (isinstance(target, ast.Boxed)
-              or (isinstance(target, (ast.Record, ast.Union)) and target.gtype_name is not None)):
+              or (isinstance(target, (ast.Record, ast.Union))
+                  and (target.gtype_name is not None or target.foreign))):
             return ast.PARAM_TRANSFER_FULL
         elif isinstance(target, (ast.Enum, ast.Bitfield)):
             return ast.PARAM_TRANSFER_NONE
@@ -400,7 +463,7 @@ usage is void (*_gtk_reserved1)(void);"""
             elif isinstance(target, (ast.Record, ast.Union)):
                 return ast.PARAM_TRANSFER_FULL
             else:
-                assert False, "Invalid constructor"
+                raise AssertionError("Invalid constructor")
         elif isinstance(target, (ast.Class, ast.Record, ast.Union)):
             # Explicitly no default for these
             return None
@@ -426,7 +489,8 @@ usage is void (*_gtk_reserved1)(void);"""
 
         param_type = options.get(OPT_TYPE)
         if param_type:
-            node.type = self._resolve(param_type.one(), node.type)
+            node.type = self._resolve(param_type.one(),
+                                      node.type, node, parent)
 
         caller_allocates = False
         annotated_direction = None
@@ -443,13 +507,10 @@ usage is void (*_gtk_reserved1)(void);"""
                     caller_allocates = '**' not in node.type.ctype
                 else:
                     caller_allocates = False
-            elif subtype == 'caller-allocates':
+            elif subtype == OPT_OUT_CALLER_ALLOCATES:
                 caller_allocates = True
-            elif subtype == 'callee-allocates':
+            elif subtype == OPT_OUT_CALLEE_ALLOCATES:
                 caller_allocates = False
-            else:
-                message.fatal("out allocation for %s is invalid (%r)" % (
-                    node, subtype))
         elif OPT_IN in options:
             annotated_direction = ast.PARAM_DIRECTION_IN
 
@@ -459,8 +520,8 @@ usage is void (*_gtk_reserved1)(void);"""
             # Also reset the transfer default if we're toggling direction
             node.transfer = self._get_transfer_default(parent, node)
 
-        transfer_tag = options.get(TAG_TRANSFER)
-        if transfer_tag:
+        transfer_tag = options.get(OPT_TRANSFER)
+        if transfer_tag and transfer_tag.length() == 1:
             node.transfer = transfer_tag.one()
 
         self._adjust_container_type(parent, node, options)
@@ -472,11 +533,9 @@ usage is void (*_gtk_reserved1)(void);"""
         if tag is not None and tag.comment is not None:
             node.doc = tag.comment
 
-        for key in options:
-            if '.' in key:
-                value = options.get(key)
-                if value:
-                    node.attributes.append((key, value.one()))
+        if options:
+            for attribute in options.getall(OPT_ATTRIBUTE):
+                node.attributes.append(attribute.flat())
 
     def _apply_annotations_annotated(self, node, block):
         if block is None:
@@ -502,7 +561,7 @@ usage is void (*_gtk_reserved1)(void);"""
 
         annos_tag = block.get(TAG_ATTRIBUTES)
         if annos_tag is not None:
-            options = AnnotationParser.parse_options(annos_tag.value)
+            options = AnnotationParser.parse_options(annos_tag, annos_tag.value)
             for key, value in options.iteritems():
                 if value:
                     node.attributes.append((key, value.one()))
@@ -524,17 +583,9 @@ usage is void (*_gtk_reserved1)(void);"""
             options = {}
         if isinstance(parent, (ast.Function, ast.VFunction)):
             scope = options.get(OPT_SCOPE)
-            if scope:
-                scope = scope.one()
-                if scope not in [ast.PARAM_SCOPE_CALL,
-                                 ast.PARAM_SCOPE_ASYNC,
-                                 ast.PARAM_SCOPE_NOTIFIED]:
-                    message.warn_node(
-                        parent,
-                        "Invalid scope %r for parameter %r" % (scope, param.argname))
-                else:
-                    param.scope = scope
-                    param.transfer = ast.PARAM_TRANSFER_NONE
+            if scope and scope.length() == 1:
+                param.scope = scope.one()
+                param.transfer = ast.PARAM_TRANSFER_NONE
 
             destroy = options.get(OPT_DESTROY)
             if destroy:
@@ -549,7 +600,7 @@ usage is void (*_gtk_reserved1)(void);"""
                     # since we don't have a way right now to flag this callback a destroy.
                     destroy_param.scope = ast.PARAM_SCOPE_NOTIFIED
             closure = options.get(OPT_CLOSURE)
-            if closure:
+            if closure and closure.length() == 1:
                 param.closure_name = self._get_validate_parameter_name(parent,
                                                                        closure.one(),
                                                                        param)
@@ -589,7 +640,7 @@ usage is void (*_gtk_reserved1)(void);"""
             if doc_name in allparams:
                 continue
             # Skip varargs, see #629759
-            if doc_name == '...':
+            if doc_name.lower() in ['...', 'varargs', TAG_RETURNS]:
                 continue
             if len(allparams) == 0:
                 text = ''
@@ -599,9 +650,11 @@ usage is void (*_gtk_reserved1)(void);"""
                 text = ', should be one of %s' % (
                 ', '.join(repr(p) for p in allparams), )
 
+            tag = block.get(doc_name)
             message.warn(
                 '%s: unknown parameter %r in documentation comment%s' % (
-                block.name, doc_name, text))
+                block.name, doc_name, text),
+                tag.position)
 
     def _apply_annotations_callable(self, node, chain, block):
         self._apply_annotations_annotated(node, block)
@@ -628,7 +681,7 @@ usage is void (*_gtk_reserved1)(void);"""
         tag = block.get(field.name)
         if not tag:
             return
-        t = tag.options.get('type')
+        t = tag.options.get(OPT_TYPE)
         if not t:
             return
         field.type = self._transformer.create_type_from_user_string(t.one())
@@ -639,14 +692,14 @@ usage is void (*_gtk_reserved1)(void);"""
         self._apply_annotations_annotated(prop, block)
         if not block:
             return
-        transfer_tag = block.get(TAG_TRANSFER)
+        transfer_tag = block.get(OPT_TRANSFER)
         if transfer_tag is not None:
             prop.transfer = transfer_tag.value
         else:
             prop.transfer = self._get_transfer_default(parent, prop)
         type_tag = block.get(TAG_TYPE)
         if type_tag:
-            prop.type = self._resolve(type_tag.value, prop.type)
+            prop.type = self._resolve(type_tag.value, prop.type, prop, parent)
 
     def _apply_annotations_signal(self, parent, signal):
         prefix = self._get_annotation_name(parent)
@@ -666,7 +719,8 @@ usage is void (*_gtk_reserved1)(void);"""
                 options = getattr(tag, 'options', {})
                 param_type = options.get(OPT_TYPE)
                 if param_type:
-                    param.type = self._resolve(param_type.one(), param.type)
+                    param.type = self._resolve(param_type.one(), param.type,
+                                               param, parent)
             else:
                 tag = None
             self._apply_annotations_param(signal, param, tag)
